/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

/* iveBitmap.c */
/* Some bitmap routines by Pat Lopresti and/or Mat Hostetter ? */
/* modified ajazar Wed Feb  8 13:39:31 EST 1995 */

#include "iveBitmap.h"

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include <malloc.h>

void 
iveBitmapZero( iveBitmap *bm )
{
  bzero( bm->data, bm->byteCount );
}

iveBitmap *
iveBitmapMake( int height, int width )
{
  iveBitmap *bm = (iveBitmap *) malloc( sizeof(iveBitmap) );
  if ( bm )
    {
      bm->byteCount =  height * width;
      bm->data = (unsigned char *) malloc( bm->byteCount );
      if ( bm->data )
	{
	  bm->height = height;
	  bm->width = width;
	}
      else
	{
	  free( bm );
	  perror("iveBitmapMake failed data malloc ");
	  return 0;
	}
      return bm;
    }
  else
    {
      perror("iveBitmapMake failed structure malloc ");
      return 0;
    }
}

iveBitmap * iveBitmapMakeCopy( iveBitmap *src )
{
  if (src)
    {
      iveBitmap *bm = ( iveBitmap * ) malloc( sizeof( iveBitmap ) );
      
      if ( bm ) 
	{
	  bm -> byteCount = src->byteCount;
	  bm -> height = src->height;
	  bm -> width = src->width;
	  bm -> data = (unsigned char *) malloc( bm->byteCount );
	  if ( ! bm->data )
	    {
	      free( bm );
	      perror("iveBitmapMakeCopy failed data malloc");
	      return 0;
	    }
	  bcopy( src->data, bm->data, bm->byteCount );
	  return bm;
	}
      else
	{
	  perror("iveBitmapMakeCopy failed data malloc");
	  return 0;
	}
    }
  else
    {
      perror("iveBitmapMakeCopy no source bitmap");
      return (0);
    }
}

int iveBitmapCopy( iveBitmap *src , iveBitmap *dest )
{
  if ((src  == NULL) ||
      (dest == NULL) || 
      (dest->width  != src->width) ||
      (dest->height != src->height))
    {
      return(0);
    }
  else
    {
      bcopy( src->data, dest->data, dest->byteCount );
      return(1);
    }
}


/* this should only be called on Make or MakeCopy results */
void iveBitmapFree(iveBitmap *bm)	
{
  free(bm->data);
  free(bm);
}

iveBitmap *
iveBitmapMakeLoad( char *file )
{
  /* reads an XBM (X bitmap) */
  FILE *fp = fopen( file, "r" );
  int c;
  int i,j;
  int width;
  int height;
  iveBitmap *bm;

  if ( fp == NULL)
    return (NULL);

  while ((c=getc(fp))!='_')
    if (c==EOF)
      {
	fclose(fp);
	return (NULL);
      }
  while ((c=getc(fp))!=' ')
    if (c==EOF)
      {
	fclose(fp);
	return (NULL);
      }
  if ((fscanf (fp,"%d",&width))==EOF)
      {
	fclose(fp);
	return (NULL);
      }
  while ((c=getc(fp))!='_')
    if (c==EOF)
      {
	fclose(fp);
	return (NULL);
      }
  while ((c=getc(fp))!=' ')
    if (c==EOF)
      {
	fclose(fp);
	return (NULL);
      }
  if ((fscanf (fp,"%d",&height))==EOF)
    {
      fclose(fp);
      return(NULL);
    }

  if ( bm = iveBitmapMake( height, width ) )
    {
      iveBitmapZero( bm );
  
      while ((c=getc(fp))!='{')
	if (c==EOF)
	  {
	    fclose(fp);
	    return (NULL);
	  }
      for ( i = 0; i < (bm->height*(bm->height+7/8)); ++i )
	{
	  unsigned int d;
	  while ((c=getc(fp))!='x')
	    if (c==EOF) 
	      return(NULL);
	  if ((fscanf(fp,"%x", &d)==EOF)) 
	    return(NULL);
	  for(j=0;j<8;j++)
	    bm->data[i] = (unsigned char) ((d&(1<<j)) ? 1 : 0) ;
	}
    }
  return bm;
}

int
iveBitmapGetBit( iveBitmap *bm, int y, int x )
{ 
  return bm->data[y * bm->width + x];
}

void
iveBitmapPutBit( iveBitmap *bm, int y, int x, int val )
{ /* void setbit(unsigned char *bitmap, int width, int x, int y, int onflag)*/
  bm->data[y * bm->width + x] = val;
}

void
iveBitmapSetBit( iveBitmap *bm, int y, int x )
{
  bm->data[y * bm->width + x] = 1;
}

void
iveBitmapUnSetBit( iveBitmap *bm, int y, int x )
{
  bm->data[y * bm->width + x] = 0;
}

void
iveBitmapWriteImage( iveBitmap *bm, FILE *stream )
{
  int row, col;
  int greyValue;

  for ( row = 0; row < bm->height; row++ )
    {
      for ( col = 0; col < bm->width; col++ )
	{
	  greyValue = iveBitmapBit(bm,row,col);
	  putc( greyValue, stream );
	}
    }
  fflush( stream );
}

void
iveBitmapReadImage( iveBitmap *bm, FILE *stream )
{
  int row, col;
  int greyValue;

  for ( row = 0; row < bm->height; row++ )
    {
      for ( col = 0; col < bm->width; col++ )
	{
	  greyValue = getc( stream );
	  iveBitmapPutBit(bm,row,col,greyValue);
	}
    }
  fflush( stream );
}

void iveBitmapReplaceValue( iveBitmap *bm, int value, int newvalue)
{
  int w = bm->width;
  int h = bm->height;
  int x,y;

  for (x=0; x<w; x++)
    for (y=0; y<h; y++)
      if (iveBitmapGetBit(bm,y,x) == value) 
	iveBitmapPutBit(bm,y,x,newvalue);
}
	    

/* ----------------------------------------------------------------- */
/* END OF CODE */
/* ----------------------------------------------------------------- */


#if 0

extern int iveBitmapSave( iveBitmap *bm, char *file );
extern int iveBitmapScale( iveBitmap *bm, iveBitmap *out );
extern void iveBitmapRotate( iveBitmap *bm,
			  iveBitmap *out,
			  int xmax,
			  int ymax,
			  double angle );

int scalebitmap (unsigned char *source,unsigned char *dest,int source_x_size,int source_y_size, int dest_x_size, int dest_y_size)
{
  int i,j,x,y,gcd_x,gcd_y,sum,temp,xreverse=0,yreverse=0,xsign=1,ysign=1;


  if (dest_x_size < 0)
    {
      xsign=-1;
      dest_x_size *= -1;
      xreverse=dest_x_size-1;
    }
  
  if (dest_y_size < 0)
    {
      ysign=-1;
      dest_y_size *= -1;
      yreverse=dest_y_size-1;
    }


  i=source_x_size;
  j=dest_x_size;
  
  while (i%j)            /* Euclid's algorithm */
    {
      temp=j;
      j=i%j;
      i=temp;
    }
  gcd_x=j;


  i=source_y_size;
  j=dest_y_size;
  
  while (i%j)            /* Euclid's algorithm */
    {
      temp=j;
      j=i%j;
      i=temp;
    }
  gcd_y=j;


  for (y=0;y<dest_y_size;++y)
    for (x=0;x<dest_x_size;++x)
      {
	sum=0;
	for (j=y*source_y_size/gcd_y;j<(y+1)*source_y_size/gcd_y;++j)
	  for (i=x*source_x_size/gcd_x;i<(x+1)*source_x_size/gcd_x;++i)
	    if (source[(int)(gcd_y*j/dest_y_size)*(int)((source_x_size+7)/8)+(int)(gcd_x*i/dest_x_size)/8] & (1<<((i*gcd_x/dest_x_size)%8))) ++sum;
	if (sum > (source_x_size/gcd_x)*(source_y_size/gcd_y) / 2)
	  dest[(ysign*y+yreverse)*(int)((dest_x_size+7)/8)+(int)((xsign*x+xreverse)/8)] |= 1<<((xsign*x+xreverse)%8);
      }
  
  return (0);
}


/* Rotates a bitmap rot radians counter-clockwise (maybe) */

#define M1 0.25
#define M2 0.75

void rotatebitmap(unsigned char *source, unsigned char *dest,
		  int xmax, int ymax, double rot)
{
  int i,j;
  double c,s,newx,newy;

  c = cos(rot);
  s = sin(rot);

  for (i=0;i<xmax;i++)
    for (j=0;j<ymax;j++)
      {
	newx = (i-xmax/2.0)*c+(j-ymax/2.0)*s+xmax/2.0;
	newy = -(i-xmax/2.0)*s+(j-ymax/2.0)*c+ymax/2.0;
	if ((newx<(xmax-1))&&(newy<(ymax-1))&&(newx>=0)&&(newy>=0)) {
	  setbit(dest,xmax,(int)(newx+M1),(int)(newy+M1),
		 getbit(source, xmax, i, j));
	  setbit(dest,xmax,(int)(newx+M2),(int)(newy+M1),
		 getbit(source, xmax, i, j));
	  setbit(dest,xmax,(int)(newx+M1),(int)(newy+M2),
		 getbit(source, xmax, i, j));
	  setbit(dest,xmax,(int)(newx+M2),(int)(newy+M2),
		 getbit(source, xmax, i, j));
	}
      }
}


int savebitmap(char *filepath, unsigned char *bitmap,int width,int height)
{
  FILE *fp;
  int i,j;


  i=strlen(filepath)-1;

  while (i>=0 && filepath[i]!='/')
    --i;

  if ((fp=fopen(filepath,"w"))==NULL)
      {
	fclose(fp);
	return (-1);
      }

  if (!strcmp(filepath+(strlen(filepath))-4,".xbm"))
      filepath[strlen(filepath)-4]='\0';

  fprintf(fp,"#define %s_width %d\n",filepath+i+1,width);
  fprintf(fp,"#define %s_height %d\n",filepath+i+1,height);
  fprintf(fp,"static char %s_bits[] = {\n",filepath+i+1);
  for (i=0;i<=(int)(height*(int)((width+7)/8));i+=12)
    {
      fprintf(fp,"   ");
      for (j=0;j<12;++j)
	{
	  if (i+j < height*(int)((width+7)/8)+(width*height%8))
	    fprintf(fp,"0x%02x",bitmap[i+j]);
	  if (i+j < height*(int)((width+7)/8)+(width*height%8)-1)
	    fprintf(fp,", ");
	}
      putc('\n',fp);
    }
  fprintf(fp,"   };\n");
  fclose(fp);
  
  return(0);
}

/* Handy routine for whoever wants it */



#endif
