/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// lite.h++

#ifndef _lite_H_
#define _lite_H_

#include <stdio.h>
#include "pfCamera.h++"
#include "pfFeatures.h++"
#include "pfPersonSize.h++"
#include "pfGesture.h++"
#include "pfSemaphore.h++"
#include "pfContour.h++"

class lite
{
private:
    static pfCamera* camera;
    static pfFeatures features;
    static pfPersonSize personSize;
    static pfGesture gestures;
    
public:
    static void initialize();
    static void interpretPfContour( pfContour& );
    static void dumpContour( FILE* );
    static void handleEvents();
    
    static inline void Project( float in[3], float out[3] ); // out is y-down
    static inline void BackProject( float in[2], float out[3], float z ); // in is y-down
    static inline void BackProject( float in[2], float out[3] ); // in is y-down
    
    static inline float* PersonDimensions() { return personSize.Size(); }
    static inline int PersonDimensionsValid() { return 1; }
    static inline float aspectRatio() { return camera->AspectRatio(); }
    
    static _feature_data kalmanFeatures;
    static _feature_data& kalmanUpdate( _feature_data& in );
    static void lite::_initializeColorClasses();
    static void lite::_reconcileWithColorClasses();
    static void lite::_reconcileColorClass(pfClassDesig ,
					   float*,
					   boolean& ,
					   float *,
					   float *);
public:
    static pfSemaphore kalmanSemaphore;
    static float mmtNoise;
    static float accelNoise;
    static int kfGrain;
    static inline _feature_data& FeatureDataKalman() { return kalmanFeatures; }
    
private:
    static int personExists; // if the contour is big enough
public:
    static inline pfGesture& getGestures() { return gestures; }
    static inline pfFeatures& getFeatures() { return features; }
    static inline int exists() { return personExists; }
    static int reportFreq;
    static int reportMove;
    static int contourGroove;
    static int printGestures;
};

inline
void
lite::Project( float in[3], float out[3] )
{
    camera -> projectToScreen( in, out );
}

inline
void
lite::BackProject( float in[2], float out[3], float z )
{
    camera -> projectScreenYtoWorldY( in, out, z );
}

inline
void
lite::BackProject( float in[2], float out[3] )
{
    BackProject( in, out, features.getCurrent().userplaneCentroid[2] );
}

#endif /* _lite_H_ */
