/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#include "matrix.h"
#include <stdio.h>

/* Creates a Matrix whose data is given. The array becomes property
 * of the matrix; it will be freed when the matrix is. The array must
 * be height*width elements long.
 */
Matrix MatrixFromArray(int height, int width, double *data)
{
  Matrix result;
  int i;

  result = Allocate(1, struct MatrixStruct);
  result->width = width;
  result->height = height;
  result->data = Allocate(height, Real*);
  result->data[0] = data;
  for(i=1;i<height;i++) {
    result->data[i] = &result->data[0][i*width];
  }
  return result;
}

/* Create a new, uninitialized matrix */
Matrix MatrixCreate(int height, int width)
{
  double *data = Allocate(width*height, Real);
  if(!data) {
    fprintf(stderr, "Cannot allocate %d by %d matrix\n", height, width);
    return NULL;
  }
  return MatrixFromArray(height, width, data);
}
