/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// pfContour.c++

#include <stream.h>
#include "pfContour.h++"

pfContour::pfContour() { clear(); }

pfContour&
pfContour::generate( iveBitmap *bm, ivePoint *pcenter, int decimate )
{
  static int horizLUT[8] = {-1, 0, 1, 1, 1, 0, -1, -1};
  static int vertLUT[8] = {-1, -1, -1, 0, 1, 1, 1, 0};

  height = bm->height;
  width = bm->width;

  clear();

  centroid[0] = pcenter->x - (pcenter->x % decimate);
  centroid[1] = pcenter->y - (pcenter->y % decimate);

  bBoxMin[0] = bBoxMax[0] = centroid[0];
  bBoxMin[1] = bBoxMax[1] = centroid[1];

#if 0
  // Find a low point on the contour
  int y;
  if( !getBit( bm, centroid[1], centroid[0] ) )
    {
      y = centroid[1]+1;
      while( y < bm->height && !getBit( bm, y, centroid[0] ) )
	y+=decimate;
      if( y == bm->height )
	return *this;
    }
  else
    {
      for( y = centroid[1];
	  y < bm->height-1 && getBit( bm, y+decimate, centroid[0] );
	  y+=decimate);
    }
  vec2iSet( points, centroid[0], y );
#endif

  // Find a good point on the side of the contour
  int fmDir = 7;
  int x = 0;
  while( x < centroid[0] && !getBit( bm, centroid[1], x ) )
    x+=decimate;
  if( x == centroid[0] )
    {
      x = bm->width-1;
      x = x - (x % decimate);
  
      while( x > centroid[0] && !getBit( bm, centroid[1], x ) )
	x-=decimate;
      if( x == centroid[0] )
	return *this;
      fmDir = 3;
    }
  vec2iSet( points, x, centroid[1] );

  int *current = points;
  length = 1;
  
  // Directions:
  //   0 1 2
  //   7 + 3
  //   6 5 4

  // int dirToBGpoint_start = 5; // DOWN (cuz we just searched up)
  int dirToBGpoint_start = fmDir;  // either left or right
  int direction;

  while( ! vec2iEquiv( current, points ) || 1==length )
    {
      int *nextPoint = current + 2;

      vec2iUpdateBox( bBoxMin, bBoxMax, current );

      // Find direction, and coords of next CW contour point 
      direction = dirToBGpoint_start;
      do {
	direction = (direction + 1) % 8;
	vec2iSet( nextPoint,
		 current[0] + decimate*horizLUT[direction],
		 current[1] + decimate*vertLUT[direction] );
	if( direction == dirToBGpoint_start )
	  {
	    cerr << "pfContour::generate: ERROR" << endl;
	    break;
	  }
      }
      while( !getBit( bm, nextPoint ) );

      // Reset the dirToBGpoint for the next time around:
      // +4 points back to current point
      // +1 rotates one more in CW dir, which is guaranteed to
      //    be in BG.
      dirToBGpoint_start = ( direction + 5 ) % 8;
      
      current = nextPoint;
      length++;
    }
  return *this;
}

