/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// pfContour.h++

#ifndef _pfContour_H_
#define _pfContour_H_

extern "C"
{
#include "iveBitmap.h"
#include "ivePoint.h"
typedef Point ivePoint;
}

class pfContour
{
  inline int getBit( iveBitmap *, int y, int x );
  inline int getBit( iveBitmap *, int *v );
public:
  const int maxContourLength = 5000;
  // these are in pixels
  int centroid[2];
  int bBoxMin[2];
  int bBoxMax[2];
  int length;
  int points[ 2 * maxContourLength ];
  inline void clear();
  int height;
  int width;
  pfContour();
  pfContour& generate( iveBitmap *, ivePoint *, int );
};

inline void vec2iZero( int *v ) { v[0] = v[1] = 0; }
inline void vec2iCopy( int *src, int *dst ) 
{
  dst[0] = src[0];
  dst[1] = src[1];
}
inline void vec2iSet( int *v, int x, int y )
{
  v[0] = x;
  v[1] = y;
}
inline void vec2iUpdateBox( int *bmin, int *bmax, int *v )
{
  if( v[0] < bmin[0] ) bmin[0] = v[0];
  if( v[1] < bmin[1] ) bmin[1] = v[1];
  if( v[0] > bmax[0] ) bmax[0] = v[0];
  if( v[1] > bmax[1] ) bmax[1] = v[1];
}
inline int vec2iEquiv( int *v1, int *v2 )
{
  return( v1[0]==v2[0] && v1[1]==v2[1] );
}

inline
int
pfContour::getBit( iveBitmap *bm, int y, int x )
{
  return( bm->data[ y * bm->width + x ] );
}

inline
int
pfContour::getBit( iveBitmap *bm, int *v )
{
  if( v[0] < 0 || v[0] >= bm->width || v[1] < 0 || v[1] >= bm->height )
    return 0;
  return( bm->data[ v[1] * bm->width + v[0] ] );
}

inline
void
pfContour::clear()
{
  length = 0;
  vec2iZero( centroid );
  vec2iZero( bBoxMin );
  vec2iZero( bBoxMax );
}


#endif // _pfContour_H_
