/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// pfGesture.h++

#ifndef _pfGesture_H_
#define _pfGesture_H_

#include <stream.h>
#include <bstring.h>
#include <math.h>
#include "pfSemaphore.h++"

class pfGesture
{
public:
    enum _handAction { handNeutral = 0,
			   handPointDown = 1,
			   handPointSide = 2,
			   handOverHead = 3,
			   handThrowing = 4 };
	    
    enum _posture { posStanding = 0,
			posCrouched = 1,
			posSitting = 2 };
		       
    enum _bodyAction { still = 0,
			   moving = 1 };

    static float bodyMoveXThreshold;		// absolute 
    static float postureCrouchThreshold;	// fraction of height
    static float postureSitThreshold;		// fraction of height

    static float handXSmall;			// absolute extension, fraction of height
    static float handXLarge;			// ""
    static float handYSmall;			// actual extension, fraction of height
    static float handYLarge;			// ""
#if 0
    static float handPDXmin;
    static float handPDYmax;
    static float handPSXmin;
    static float handPSYmin;
    static float handPSYmin;
#endif
    pfSemaphore semaphore;

private:

    _bodyAction bodyAction;
    _posture posture;
    _handAction leftHandAction;
    _handAction rightHandAction;

    float headSave[9];
    float lhandSave[9];
    float rhandSave[9];
    float centSave[9];

public:

    pfGesture();
    void Clear();
    void Update( float* hd, float* lh, float* rh, float* c, float height );

    inline int LeftHandAction() { return leftHandAction; }
    inline int RightHandAction() { return rightHandAction; }
    inline int Posture() { return posture; }
    inline int BodyAction() { return bodyAction; }

    inline int extendedLeft() { return( handPointDown == leftHandAction ); }
    inline int extendedRight() { return( handPointDown == rightHandAction ); }
    inline int pointingLeft() { return( handPointSide == leftHandAction ); }
    inline int pointingRight() { return( handPointSide == rightHandAction ); }
    inline int handMovingLeft() { return 0; }
    inline int handMovingRight() { return 0; }
    //
    //
    inline int throwLeft() { return( handThrowing == leftHandAction ); }
    inline int throwRight() { return( handThrowing == rightHandAction ); }
    inline int standing() { return( posStanding == posture ); }
    inline int sitting() { return( posSitting == posture ); }
    inline int bendingOver() { return( posCrouched == posture ); }
    inline int lhandOverHead() { return( handOverHead == leftHandAction ); }
    inline int rhandOverHead() { return( handOverHead == rightHandAction ); }

    inline int bodyMoving() { return( moving == bodyAction ); }

    ostream& Print( ostream& );
};

inline
void
vector9Copy( float* src, float* dst )
{
    bcopy( src, dst, 9*sizeof(float) );
}

inline
void
vector9Zero( float* v )
{
    for( int i = 0; i < 9; i++ )
       v[i] = 0.;
}

#endif /* _pfGesture_H_ */

