/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// pfPersonSize.h++

#ifndef _pfPersonSize_H_
#define _pfPersonSize_H_

#include <stream.h>

class pfPersonSize
{
  static float defVarHead;
  static float defAlpha;
  static int defMissThreshold;

  float meanHead;	// current robust mean of head height
  float varHead;	// perunit error threshold for mmt rejection
  float alpha;		// blending factor for accepted mmts
  int missCount;	// count of consecutive mmt rejections
  int missThreshold;	// threshold for rejection count
  float size[3];	// "size" ( width, height, ?? )
public:
  pfPersonSize( float h = 0. );
  inline float standingHeadSample( float h );
  inline float value() { return meanHead; }
  inline float* Size() { size[0]=.5*meanHead; size[1]=meanHead; return size; }
};

inline
float
pfPersonSize::standingHeadSample( float h )
{
  // we assume this is a valid mmt of the person`s head height
  // i.e. test for personExists and personMoving are pre-req
  // for this call --AJA
  float error = h - meanHead;
  float absError = ( error < 0. ) ? -error : error;
  float perunitError = (meanHead <= absError) ? 1. : (absError / meanHead);
  if( perunitError < varHead )
     { // "h" is a valid mmt of the current head height
       meanHead = ( 1. - alpha ) * meanHead + alpha * h;
       missCount = 0;
     }
  else
     { // "h" is noise, or a valid mmt of a new head
       missCount++;
       if( missCount > missThreshold )
	  { // new head
	    cerr << endl << "***** NEW HEAD *****" << endl << endl;
	    meanHead = h;
	    missCount = 0;
	  }
     }
  return meanHead;
}

#endif /* _pfPersonSize_H_ */
