/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

/* pfinder-main.c++ */

/* RCS: $Header: /v/projects/pfinder/src/RCS/pfinder-main.c++,v 9.0 1997/03/08 15:56:57 cwren Exp $ */

#include <stdio.h>
#include <stdlib.h>
#define _BSD_SIGNALS
#include <signal.h>
#include <unistd.h>
#include <iostream.h>
#include <bstring.h>
#include <math.h>

#include "VLImage.h++"
#include "pfGesture.h++"
#include "pfContour.h++"
#include "pfTextureMap.h++"

#define Time MYTIME
#include "lite.h++"
#undef Time

extern "C"
{
#include "ivePoint.h"
#include "iveBitmap.h"
#include "iveImage.h"
#include "bsXclient.h"
#include "bsColor.h"
#include "bsClassify.h"
#include "bsSegment.h"
#include "pfMorphology.h"
}

static int bgMode = 0;  	// save bg to bgstream
static int saveMode = 0;	// save output to savestream
static int readMode = 0;	// read input from stdin
static int pauseMode = 0;	// loop on current image
static int dumpMode = 0;	// dump input to dumpstream
static int statMode = 0;	// dump class stats to statstream
static int contourMode = 0;	// dump contour to contourstream
static int editMode = 0;	// allow classes editing
static int expert = 0;		// show all the gory sliders
static int dumpCount = 0;
static int dumpDelay = 0;
static int texture_mode = 0;     /* allow texture map of person
				    to be sent */
static char *chroma = NULL;
static char *dumpFile = NULL;
static char *statFile = NULL;
static char *saveFile = NULL;
static char *contourFile = NULL;
static char *bgFile = NULL;
static FILE *dumpstream = stdout;
static FILE *savestream = stdout;
static FILE *statstream = stdout;
static FILE *contourstream = stdout;
static FILE *bgstream = NULL;
static int decimate = 1;
static int useALIVEInterface = 1;
static int useXInterface = 1;
static int verbose = 0;
static int maxForegroundClasses = 8;
static int numberOfBGFrames = 40;
static iveImage **BackgroundFrames = NULL;
static iveImage **imageBuffers = NULL;
Point lowerLeft;
Point lowerRight;

pfTextureMap *pf_global_texture_map = NULL;  // outgoing texture map

extern "C" {
Point seed_points_for_morphology[1024];
int erodemultiple = 10;
}

int THAD_TRACKER = 0; /* track green and red blobs for thad */
int INITCLASSFLAG = 0;

static int EDIT = 0;	// edit classes NOW

static int MORPHOLOGY = 3;
static int CONTOUR=1;
extern int FOREGROUND;  // should be in bsClassify.h (or something)
static int HOWSLOW = 1;
static int FRAMEZERO = 0;
static int SHOWSEEDS=0;

int SEGMENT=1; /* continous */
int SEGMENT_TRIGGER=0; /* one shot */
static int EVILBITMAPS=1;

void cleanup( void );
void cleanup_sig( ... );

extern void startRpcServer(void);
extern void stopRpcServer(void);
void call_cached_pf_reqests(void);



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * howslow
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static const int NFRAMES = 25;

static void howslow() 
{
  static int total = -1;
  static int count = 0;
  static struct timeval start;
  static struct timeval end;
  static double rate;
  static double time_sec;

  total++;
  // count = number of calls since init, *including* this one
  // Thusly, when count == NFRAMES, the time elapsed is for NFRAMES
  //  calls to processFrame()

  if (count==0)  // only happens once
     {
	gettimeofday(&start);
	count = 1;
	return;
     } 
  else if ( count++ == NFRAMES ) 
     {
	gettimeofday( &end );
	time_sec = (end.tv_usec - start.tv_usec)/1000000.0 + 
	   (end.tv_sec - start.tv_sec);
	rate = (double) NFRAMES / time_sec; // frames per second
	// fprintf ( stderr, "[%2.2f Hz]\n", rate );
	fprintf ( stderr, "[%2.2f Hz, (%d frames, %d total, %.6f sec) %d]\n", 
		  rate, NFRAMES, (int)(30.*time_sec), time_sec, total );
	start = end;
	count = 1;
	return;
     } 
  // putc('.',stderr);fflush(stderr);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * image flow control
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static int STEP = 1;

void bsMainSetPause(int newvalue) {
  pauseMode = newvalue;
}

int bsMainGetPause() {
  return pauseMode;
}

void bsMainStep()
{
  STEP =1;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * clickframe
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static int CLICKME=0;

void
bsMainDumpNextFrame()
{
  CLICKME =1;
}

void
bsMainEditNextFrame()
{
  if (editMode)
    EDIT =1;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * processFrame
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

static int frameNumber = -1;

int bsMainGetFrameNumber()
{
  return frameNumber;
}

void
processFrame( iveImage *image)
{
  /* does not need frameNumber as an arg; can keep track internally */

  int j, nums;
  static Point centroid;
  static iveBitmap *newMap = (iveBitmap *)NULL;
  static iveBitmap *oldMap = (iveBitmap *)NULL;
  iveBitmap *tempMap = NULL;
#ifdef DONOTHING
  howslow(); return;
  /* NOTREACHED */
#endif
  frameNumber++;

  if (FRAMEZERO) {
    frameNumber = 0;
    FRAMEZERO = 0;
  }  

  if( newMap == (iveBitmap *)NULL )
    {
      int height = image->height;
      int width = image->width;
      
      newMap = iveBitmapMake( height, width );
      oldMap = iveBitmapMake( height, width );
      iveBitmapZero( oldMap );
      centroid.x = 0;
      centroid.y = 0;	
    }
  
  if( dumpMode )
    {
      if( HOWSLOW ) howslow();
      if( dumpCount>0 && frameNumber >= dumpCount )
	{
	  if( imageBuffers )
	    {
	      for( int i = 0; i < dumpCount; i++ )
		iveImageWriteRaw( imageBuffers[i], dumpstream );
	    }
	  cleanup();
	}
      if( verbose )
	cerr << "pfinder: dumping frame " << frameNumber << endl;
      if( useXInterface )
	{
	  bsXDisplayImage( image, 1);
	}
      if( imageBuffers )
	iveImageCopy( image, imageBuffers[frameNumber] );
      else
	iveImageWriteRaw( image, dumpstream );

      if( statMode )
	{	
	  bsClassifyPrintClasses(image->width,image->height,statstream);
	}		

      if( contourMode )
	{	
	  lite::dumpContour(contourstream);
	}		
  
      if ( saveMode ) 
	{
	  iveBitmapWriteImage( newMap, savestream );
	}
      return;
    }
  

  if (frameNumber < numberOfBGFrames) {
    
    BackgroundFrames[frameNumber] = iveImageMakeCopy(image);
    
    if( useXInterface ) {
      bsXDisplayImage( image , 1);
    }
    
    if( statMode )
      {	
	bsClassifyPrintClasses(image->width,image->height,statstream);
      }		
    
    if( contourMode )
      {	
        lite::dumpContour(contourstream);
      }		

    if ( saveMode ) 
      {
	iveBitmapWriteImage( newMap, savestream );
      }
    return;
  }

  if (frameNumber == numberOfBGFrames) 
    {
      if (!chroma) 
	{
	  bsClassifyMakeBackgroundClasses(BackgroundFrames, numberOfBGFrames);
	} else {
	  bsClassifyChromakey(chroma);
	}
      if ( bgMode ) 
	{
	  bsClassifyWriteBackgroundClasses( bgstream );
	}
    }
  
  /* classify */

  if( HOWSLOW ) howslow();
  int dosegment = SEGMENT || SEGMENT_TRIGGER;
  SEGMENT_TRIGGER = 0;
  if (dosegment)     
    {
      int RASTERPAINT = 1;
      
      if( useXInterface && EVILBITMAPS && SHOWSEEDS ) 
	{
	  bsXNewCanvas( 1 );  
	  bsClassifyDisplayClasses(1);
	}

      iveBitmapZero( newMap );
      bsSegmentFindSilhouette( image, newMap, oldMap, 
			      decimate, &(centroid));

      if (FOREGROUND) bsClassifySetForegroundStats(image, 
						   newMap);
    
      
      bsClassifyUpdateBG( image, newMap );
	
      if( useXInterface && EVILBITMAPS ) 
	{
	  if (!SHOWSEEDS) bsXNewCanvas( 1 );  
	  
	  if (!RASTERPAINT)
	    bsXCommitPaint( 1 );

	}
      
  
      if (MORPHOLOGY) {
	      for (j = 0; 
		   seed_points_for_morphology[j].x && 
		     seed_points_for_morphology[j].y ; 
		   j++) ;
	      nums = j;
	      pfMorphBridgeGaps(newMap, 
				seed_points_for_morphology, nums , 
				decimate, MORPHOLOGY);
      }
      if( useXInterface && EVILBITMAPS ) 
	{
	  if (RASTERPAINT)
	    bsXDisplayBitmap( newMap , 1);
	  else
	    bsXCommitPaint( 1 ); 
	  
	}
    } else {
      if( useXInterface && EVILBITMAPS)
	bsXDisplayImage(image, 1);
    }
  
  if (dosegment && CONTOUR) {
      static pfContour contour;
      contour.generate( newMap, &centroid, decimate );

      lowerLeft.x = contour.bBoxMin[0];
      lowerLeft.y = contour.bBoxMax[1];
      lowerRight.x = contour.bBoxMax[0];
      lowerRight.y = contour.bBoxMax[1];
      if (!editMode)
	lite::interpretPfContour( contour );

      if( useXInterface ) {
	  bsXDisplayContour( contour.points, contour.length, 0 );
	  bsXSync();	
      }
      if( useALIVEInterface ) {
	  /* sync requests are only replied to when
	     segment and contour are on */
	  call_cached_pf_reqests();
      }
      if (texture_mode)
	{
	  pf_global_texture_map->copyFromIveImage(image,
						  contour.bBoxMin[0],
						  contour.bBoxMin[1],
						  contour.bBoxMax[0],
						  contour.bBoxMax[1]);
	}
  }

  if( statMode )
    {	
      bsClassifyPrintClasses(image->width,image->height,statstream);
    }		
  
  if( contourMode )
    {	
      lite::dumpContour(contourstream);
    }		

  if ( saveMode ) 
    {
      iveBitmapWriteImage( newMap, savestream );
    }

  if ( CLICKME )
    {
      if (verbose)
	cerr << "dumping this frame's data" << endl; 

      CLICKME = 0;
      if (statFile != (char *) NULL)
	bsClassifyPrintClasses(image->width,image->height, statstream);
      if (contourFile != (char *) NULL)
	lite::dumpContour(contourstream);
      if (dumpFile != (char *) NULL)
	iveImageWriteRaw( image, dumpstream );
      if (saveFile != (char *) NULL)
	iveBitmapWriteImage( newMap, savestream );
    }


  if(EDIT)
    {
      bsXEditClasses(image, newMap, maxForegroundClasses);
      EDIT = 0;
    }

  tempMap = oldMap;
  oldMap = newMap;
  newMap = tempMap;
}

 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * cleanup
 * cleanup_sig
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void cleanup_do( void ) {
      
      if (useALIVEInterface) {
	    cerr << getpid() << " stopping rpc server" << endl;
	    stopRpcServer();
      }

      if ( useXInterface ) {
	    cerr << getpid() << " cleaning up X" << endl;
	    bsXClose();
      }

      if (!readMode) {
	    cerr << getpid() << " cleaning up VL" << endl;
	    VLImage::Close();
      }
}

void cleanup( void )
{
  cleanup_do();

  exit(1);
}

void cleanup_sig( ... )
{
  cerr << "Caught Signal" << getpid()  << endl;       
  cleanup();
}

static void 	FrameZero() { FRAMEZERO = 1; }
static int  bsMainGetContour() { return CONTOUR; }
static void bsMainSetContour(int newvalue) { CONTOUR = newvalue; }
static float  bsMainGetMorphology() { return (float) MORPHOLOGY; }
static void bsMainSetMorphology(float newvalue) { MORPHOLOGY = (int) newvalue; printf("morph %d\n", MORPHOLOGY);}
static int  bsMainGetEvilbitmaps() { return EVILBITMAPS; }
static void bsMainSetEvilbitmaps(int newvalue) { EVILBITMAPS = newvalue; }
static int  bsMainGetSegment() { return !SEGMENT; }
static void bsMainSetSegment(int newvalue) { SEGMENT = !newvalue; }

void verboseCallback( void * )
{
  bsColorSetVerbose( 1 );
  bsClassifySetVerbose( 1 );
  VLImage::Verbose( 1 );
}

void thadModeCallback( void * )
{
  THAD_TRACKER = !THAD_TRACKER;
  if (THAD_TRACKER) maxForegroundClasses =3;
}

void blackModeCallback( void * )
{
  bsColorIgonreBlack();
}

void chrisModeCallback( void * )
{
  expert =1;
  MORPHOLOGY=0;
  bsSegmentSetThresh(1500.0);
  bsColorSetNormalFloor(0.70);
  bsClassifySetThresh(200.0);
}

void chromaModeCallback( void * )
{
  bsSegmentSetThresh(10.0);
}

void dumpModeCallback( void * )
{
  dumpMode = 1;
  readMode = 0;
  saveMode = 0;
  maxForegroundClasses =1;
}

void readModeCallback( void * )
{
  readMode = 1;
}

void saveModeCallback( void * )
{
  dumpMode = 0;
  saveMode = 1;
}

void aliveCallback (void *)
{
  bsMainSetEvilbitmaps(0);
  maxForegroundClasses = 1;
  bsClassifySetUpdateBG(0.01);
  expert = 0;
}  

void expertCallback (void *)
{
  bsMainSetEvilbitmaps(1);
  maxForegroundClasses = 8;
  expert = 1;
}  


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * main
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


void
help( char *progname )
{
  static char *HelpString = "\n"\
    "\t-alive                 ALIVE Demo mode\n"\
    "\t-expert                Expert Mode (anti-ALIVE)\n"\
    "\t-n                     Toggle whether to use ALIVE interface\n"\
    "\t-v                     Main program verbose mode on\n"\
    "\t-v                     bsColor and VL modules verbose\n"\
    "\t-X                     Use X Interface\n"\
    "\t-b <int>               Number of BG Frames\n"\
    "\t-d <int>               Decimation factor\n"\
    "\t-D <int>               Number of frames to dump to stdard out\n"\
    "\t-M <int>               Dilate Erode factor\n"\
    "\t-T                     Thad mode: track green and red blobs\n"\
    "\t-ignoreblack           ignore black pixels background calculations\n"\
    "\t-chris                 Chris Mode: process my shitty data\n"\
    "\t-cont                  save contours to cout\n"\
    "\t-stat                  Stats: blob centers to cou\n"\
    "\t-R                     images fm cin\n"\
    "\t-single                loop on first captured image\n"\
    "\t-edit                  allow class editing\n"\
    "\t-seeds                 mark zoomgrow seeds on image\n"\
    "\t-S                     bitmaps to cout\n"\
    "\t-bg                    save bg stats to bgfile\n"\
    "\t-texture               turn on sending of texture maps\n"\
    "\t-E <int>               erode multiple\n"\
    "\t-w <int>               Width of images\n"\
    "\t-h <int>               Height of images\n"\
    "\t-size <int> <int>      height width of images\n"\
    "\t-z <int>               Zoom factor\n"\
    "\t-f <int>               Max number of foreground classes\n"\
    "\t-in <int>              index of video source\n"\
    "\t                          0: INDY_COMPOSITE\n"\
    "\t                          1: INDY_SVIDEO\n"\
    "\t                          0: GALILEO_SVIDEO_1\n"\
    "\t                          1: GALILEO_SVIDEO_2\n"\
    "\t                          2: GALILEO_SVIDEO_3\n"\
    "\t                          3: GALILEO_COMPOSITE_1\n"\
    "\t                          4: GALILEO_COMPOSITE_2\n"\
    "\t                          5: GALILEO_COMPOSITE_3\n"\
    "\t                          6: GALILEO_YRB_1\n"\
    "\t                          7: GALILEO_YRB_2\n"\
    "\t-nb <int>              number of VL buffers\n"\
    "\t-noslow                Turn off rate reporting\n"\
    "\t-help                  Prints usage (this message)\n"\
    "\t-dumpfile <string>     Filename to dump input to instead of cout\n"\
    "\t-statfile <string>     Filename to save blob stats to instd. of cout\n"\
    "\t-contourfile <string>  Filename ot save contours to instead of cout\n"\
    "\t-avefile <string>      Filename to save segments to instead of cout\n"\
    "\t-bgfile <string>       Filename to save background classes\n"\
    "\t-chroma <string>       use chromakey instead of aquiring background \n"\
    "\t                        (key on %2x%2x%2x)\n";

  cerr << "usage: " << progname << " [options]" << HelpString; 
  cleanup();
}

int 
main(int argc, char *argv[])
{
  int zoom = 4;

  int width = 640;
  int height = 480;
  int nbuffers = 3;
  int input_connector = VLImage::INDY_SVIDEO;

  for (int idx=1; idx<argc; idx++)
    {
      if (!strcmp(argv[idx],"-alive"))
	{
	  aliveCallback(0);
	}
      else if (!strcmp(argv[idx],"-expert"))
	{
	  expertCallback(0);
	}
      else if (!strcmp(argv[idx],"-n"))
	{
	  useALIVEInterface = !useALIVEInterface;
	}
      else if (!strcmp(argv[idx],"-v"))
	{
	  verbose = 1;
	  verboseCallback(0);
	}
      else if (!strcmp(argv[idx],"-X"))
	{
	  useXInterface = !useXInterface;
	}
      else if (!strcmp(argv[idx],"-b"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      numberOfBGFrames = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -b <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-d"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      decimate = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -d <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-D"))
	{
	  dumpModeCallback(0);
	  idx++;
	  if (idx<argc)
	    {
	      dumpCount = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -D <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-M"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      MORPHOLOGY = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -M <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-T"))
	{
	  thadModeCallback(0);
	}
      else if (!strcmp(argv[idx],"-ignoreblack"))
	{
	  blackModeCallback(0);
	}
      else if (!strcmp(argv[idx],"-chris"))
	{
	  chrisModeCallback(0);
	}
      else if (!strcmp(argv[idx],"-cont"))
	{
	  contourMode = !contourMode;
	}
      else if (!strcmp(argv[idx],"-stat"))
	{
	  statMode = !statMode;
	}
      else if (!strcmp(argv[idx],"-R"))
	{
	  readModeCallback(0);
	}
      else if (!strcmp(argv[idx],"-single"))
	{
	  pauseMode = !pauseMode;
	}
      else if (!strcmp(argv[idx],"-edit"))
	{
	  editMode = !editMode;
	}
      else if (!strcmp(argv[idx],"-seeds"))
	{
	  SHOWSEEDS = !SHOWSEEDS;
	}
      else if (!strcmp(argv[idx],"-S"))
	{
	  saveModeCallback(0);
	}
      else if (!strcmp(argv[idx],"-texture"))
	{
	  texture_mode = !texture_mode;
	}
      else if (!strcmp(argv[idx],"-savebg"))
	{
	  bgMode = !bgMode;
	}      
      else if (!strcmp(argv[idx],"-E"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      erodemultiple = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -E <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-w"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      width = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -w <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-h"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      height = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -h <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-size"))
	{
	  if ((idx+2)<argc)
	    {
	      idx++;
	      height = atoi(argv[idx]);
	      idx++;
	      width = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument(s) for -size <int> <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-z"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      zoom = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -z <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-f"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      maxForegroundClasses = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -f <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-in"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      input_connector = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -in <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-nb"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      nbuffers = atoi(argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -nb <int>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-noslow"))
	{
	  HOWSLOW = !HOWSLOW;
	}
      else if (!strcmp(argv[idx],"-help"))
	{
	  help(argv[0]);
	}
      else if (!strcmp(argv[idx],"-dumpfile"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      dumpFile = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -dumpfile <string>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-bgfile"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      bgFile = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -bgfile <string>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-statfile"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      statFile = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -statfile <string>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-contourfile"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      contourFile = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -contourfile <string>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-savefile"))
	{
	  idx++;
	  if (idx<argc)
	    {
	      saveFile = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -savefile <string>" << endl;
	      cleanup();
	    }
	}
      else if (!strcmp(argv[idx],"-chroma"))
	{
	  chromaModeCallback(0);
	  idx++;
	  if (idx<argc)
	    {
	      chroma = (argv[idx]);
	    } 
	  else 
	    {
	      cerr << "missing argument for -chroma <string>" << endl;
	      cleanup();
	    }
	}
      else
	{
	  cerr << "unrecognized argument (try -help) :" << argv[idx] << endl;
	  cleanup();
	}	  
    }

  signal(SIGINT, cleanup_sig);
  signal(SIGPIPE, cleanup_sig); 
  signal(SIGHUP, cleanup_sig);
  signal(SIGILL, cleanup_sig);

  if (chroma)
    {
      numberOfBGFrames = 0;
    }

  if (!bgFile)
    {
      bgFile = "/tmp/pfinder.bg";
    }
  if (bgMode)
    {
      if (verbose)
	cerr << "bgFile was " << bgFile << endl;
      if ((bgstream = fopen(bgFile, "w")) == (FILE *) NULL)
	{
	  cerr << "Error opening bg file" << endl;
	  cleanup();
	}
    }
  
  if (dumpFile)
    {
      if (verbose)
	cerr << "dumpFile was " << dumpFile << endl;
      if ((dumpstream = fopen(dumpFile, "w")) == (FILE *) NULL)
	{
	  cerr << "Error opening dump file" << endl;
	  cleanup();
	}
    }

  if (statFile)
    {
      if (verbose)
	cerr << "statFile was " << statFile << endl;
      if ((statstream = fopen(statFile, "w")) == (FILE *) NULL)
	{
	  cerr << "Error opening stat file" << endl;
	  cleanup();
	}
    }

  if (contourFile)
    {
      if (verbose)
	cerr << "contourFile was " << contourFile << endl;
      if ((contourstream = fopen(contourFile, "w")) == (FILE *) NULL)
	{
	  cerr << "Error opening contour file" << endl;
	  cleanup();
	}
    }


  if (saveFile)
    {
      if (verbose)
	cerr << "saveFile was " << saveFile << endl;
      if ((savestream = fopen(saveFile, "w")) == (FILE *) NULL)
	{
	  cerr << "Error opening save file" << endl;
	  cleanup();
	}
    }


  if( !readMode )
    {
      VLImage::NBuffers( nbuffers );
#ifdef DIGITIZEYUV
      VLImage::Encoding( VLImage::YUV );
#else
      VLImage::Encoding( VLImage::RGB );
#endif
      VLImage::Zoom( zoom );
      VLImage::Input( input_connector );
      VLImage::Init();
      height = VLImage::GetHeight();
      width = VLImage::GetWidth();
    }
  if( dumpMode )
    {
      if( verbose )
	cerr << "pfinder: VL frame size: height= " << height
	     << " width= " << width << endl;
      if( dumpMode && dumpCount <= 100 && dumpCount>0)
	{
	  imageBuffers = (iveImage **) malloc( dumpCount*sizeof(iveImage*) );
	  if( imageBuffers )
	    {
	      for( int i = 0; i < dumpCount; i++ )
		{
		  imageBuffers[i] = iveImageMake( height, width );
		  if( ! imageBuffers[i] )
		    {
		      fprintf( stderr, "pfinder: failed to malloc image %d\n", i );
		      exit( 1 );
		    }
		}
	    }
	}
    }
  
  lite::initialize();

  if( !dumpMode )
     {
       bsClassifyInitialize(width, height, maxForegroundClasses);
       
       BackgroundFrames = (iveImage **) malloc( sizeof(iveImage *) * 
					       numberOfBGFrames );
     }
  
  if( useXInterface )
    {
      bsXInitialize(width, height, 
		    maxForegroundClasses, 
		    0, NULL );
      
      if ( !dumpMode ) 
	{
	  bsXMakeSliderProc( bsSegmentSetThresh, "FG/BG Threshhold",
			    bsSegmentGetThreshMax(), 0.0, 
			    bsSegmentGetThresh() );
	  bsXMakeSliderProc( bsColorSetNormalFloor, "Normalize Floor",
			    1.0, 0.0, 
			    bsColorGetNormalFloor() );
	  if (expert)
	    {
	      bsXMakeSliderProc( bsClassifySetThresh, "Sub-Class Threshhold",
				bsClassifyGetThreshMax(), 0.0, 
				bsClassifyGetThresh() );

	      bsXMakeSliderProc( bsMainSetMorphology, "Dilation",
				10.0, 0.0, bsMainGetMorphology() );
	    }
	}
      
      if (expert)
	{
	  bsXMakeSliderProc( bsXSetThresh, "Display Contrast",
			    1.0, 0.0, bsXGetThresh() );
	}
      
      if ( !dumpMode ) 
	{
	  bsXMakeButtonProc(bsMainSetSegment, "Pass-Through",
			    bsMainGetSegment() );
	  if (expert)
	    {
	      bsXMakeButtonProc(bsMainSetContour, "Compute Contour",
				bsMainGetContour() );
	    }
	  bsXMakeButtonProc(bsMainSetEvilbitmaps, "Display Segmentation",
			    bsMainGetEvilbitmaps() );

	  if (expert)
	    {
	      bsXMakeSliderProc( pfFeatures::setHandMin, "Hand min",
				1., 0., pfFeatures::getHandMin() );
	      bsXMakeSliderProc( pfFeatures::setHandMax, "Hand max",
				1., 0., pfFeatures::getHandMax() );
	    }

	  if (expert)
	    {
	      bsXMakeSeparator();
	      bsXMakeButtonProc(bsSegmentSetZoomGrow, "Grow",
				bsSegmentGetZoomGrow() );
	      bsXMakeButtonProc(bsSegmentSetRecursive, "Zoom",
				bsSegmentGetRecursive() );
	      bsXMakeButtonProc(bsSegmentSetSmooth, "Smooth",
				bsSegmentGetSmooth() );
	    }
	  
	  if (expert)
	    {
	      bsXMakeSeparator();
	      bsXMakeSliderProc(bsClassifySetUpdateBG, "Update Percentage",
				1.0, 0.0,
				bsClassifyGetUpdateBG() );
	      bsXMakeSliderProc(bsColorSetUpdateBG, "Update Rate",
				1.0, 0.0,
				bsColorGetUpdateBG() );
	    }
	  
	  if (expert) 
	    {
	      bsXMakeSeparator();
	      bsXMakeButtonProc(bsColorSetNormalize, "Normalize Enable",
				bsColorGetNormalize() );
	      bsXMakeButtonProc(bsClassifySetNormalize, "Normalize everywhere",
				bsClassifyGetNormalize());
	    }
	  bsXMakeButtonProc(bsClassifySetForeground, "Use Foreground",
			    bsClassifyGetForeground() );
	  
	  bsXMakeSeparator();
	  if (expert)
	    {
	      bsXMakeButtonProc( bsMainSetPause, "Pause",
				bsMainGetPause());
	      bsXMakePushButton( bsMainStep, "Step" );
	      bsXMakeSeparator();
	      bsXMakePushButton( bsMainDumpNextFrame, "Dump next frame" );
	    }
	  bsXMakePushButton( FrameZero, "Reaquire BG" );
	  
	}
      
      if (editMode)
	bsXMakePushButton( bsMainEditNextFrame, "edit color classes" );

      bsXMakePushButton( cleanup, "panic: core dump" );
    }

  if (texture_mode)
    {
      pf_global_texture_map = new pfTextureMap(width, height);
    }

  if (useALIVEInterface) startRpcServer();

  if( readMode )		// read from a file
    {
      iveImage *image = iveImageMake( height, width ); 
      
      for(;;)
	 {
	   if( useXInterface ) 
	     bsXHandleEvents();
	   
	   if (pauseMode && !STEP)
	     {
	       processFrame( image );
	     }
	   else
	     {
	       if (iveImageReadRaw( image, stdin ))
		 {
		   fprintf(stderr,"end of input\n");
		   cleanup();
		 }
	       processFrame( image );
	     }
	   STEP = 0;
	 }
    }
  else				// live or dumpMode
    {
      sleep( dumpDelay );

#ifdef DIGITIZEYUV
      VLYuvImage picture;
#else
      VLRgbImage picture;
#endif
      iveImage wrapper;
      for(;;)
	 {
	   if( useXInterface )
	      bsXHandleEvents();
      
	   if (pauseMode && !STEP)
	     {
	       processFrame( &wrapper );
	     }
	   else
	     {
	       picture.Grab();
	       
	       iveImageInit(&wrapper, 
			    picture.Height(), 
			    picture.Width(), 
			    picture );
	       processFrame( &wrapper );
	       
	       picture.Free();
	     }

	   STEP = 0;

	 }
    }      
}
