/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#define LIITE

#include <stream.h>
#include <rpc/rpc.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <signal.h>
#include <sys/time.h>
#include <unistd.h>
#include <limits.h>
#include <bstring.h>

#include "pfinder_svc.h"
#include "lite.h++"

static const float ALSCALE = 1.5;

extern void cache_pf_request(pfopts *opts,struct svc_req *req);

// ------------------------------------------------
// utility functions

inline
void
dv3fset( pfv3f& a, float* b, pfopts *opts )
{
    a.x = b[0];
    a.y = b[1];
    a.z = b[2];
    if( opts && !opts->alive_coords )
       {
	   a.x *= ALSCALE;
	   a.y *= ALSCALE;
	   a.z *= ALSCALE;
       }
}

inline
void
dv2fset( pfv2f& a, float* b )
{
    a.x = b[0] - .5;
    a.y = b[1] - .5;
    a.y *= lite::aspectRatio(); // height/width
}

// note this module depends on the version 1 module for certain
// functionality, it cannot exits on it's own.  

extern int Gaze_valid;
extern pfv3f Gaze;

extern pfgest getg( pfopts *);
extern pfcont2d getc2(pfopts *);
extern pfcont3d getc3(pfopts *);
extern pftexture gettexture(pfopts *);

static pfinfof3 getf3( pfopts *opts)
{
    pfinfof3 r;
    static pfinfof3_s rfeats;
    static pfinfof3_e parts[MAXFEATS];
    r.pfinfof3_u.feats = NULL;

    rfeats.parts.parts_len = MAXFEATS;
    rfeats.parts.parts_val = parts;

    if (!(r.opt = opts->f3)) return r;
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteLOCK();
    else
       lite::getFeatures().semaphore.remoteLOCK();

    
    _feature_data& fts = 
      (opts->kalman) 
	? (lite::FeatureDataKalman()) 
	  : (lite::getFeatures().getLatest());
    
    dv3fset( parts[pfrpc_CENTROID].pos,  fts.userplaneCentroid,  opts);
    dv3fset( parts[pfrpc_RIGHTHAND].pos, fts.userplaneRightHand, opts);
    dv3fset( parts[pfrpc_LEFTHAND].pos,  fts.userplaneLeftHand,  opts);
    dv3fset( parts[pfrpc_HEAD].pos,      fts.userplaneHead,      opts);
    dv3fset( parts[pfrpc_RIGHTFOOT].pos, fts.userplaneRightFoot, opts);
    dv3fset( parts[pfrpc_LEFTFOOT].pos,  fts.userplaneLeftFoot,  opts);

    parts[pfrpc_CENTROID].valid  = TRUE;
    parts[pfrpc_RIGHTHAND].valid = fts.rightHandValid;
    parts[pfrpc_LEFTHAND].valid  = fts.leftHandValid;
    parts[pfrpc_HEAD].valid      = fts.headValid;
    parts[pfrpc_RIGHTFOOT].valid = fts.rightFootValid;
    parts[pfrpc_LEFTFOOT].valid  = fts.leftFootValid;

    if (Gaze_valid) {
	rfeats.gaze_pos = Gaze;
    } else {
#define TMPGAZE
#ifdef TMPGAZE
	rfeats.gaze_pos.y = rfeats.gaze_pos.z = 0.0;
	rfeats.gaze_pos.x = (parts[pfrpc_RIGHTHAND].pos.x - 
			     parts[pfrpc_CENTROID].pos.x);
#else
	rfeats.gaze_pos.x = rfeats.gaze_pos.y = rfeats.gaze_pos.z = 0.0;
#endif
    }
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteUNLOCK();
    else
       lite::getFeatures().semaphore.remoteUNLOCK();

    r.pfinfof3_u.feats = &rfeats;
    return r;
}

static pfinfof2 getf2( pfopts *opts)
{
    pfinfof2 r;
    static pfinfof2_s rfeats;
    static pfinfof2_e parts[MAXFEATS];
    r.pfinfof2_u.feats = NULL;

    rfeats.parts.parts_len = MAXFEATS;
    rfeats.parts.parts_val = parts;

    if (!(r.opt = opts->f2)) return r;
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteLOCK();
    else
       lite::getFeatures().semaphore.remoteLOCK();

    _feature_data& fts = 
       (opts->kalman) 
       ? (lite::FeatureDataKalman()) 
       : (lite::getFeatures().getLatest());

    dv2fset( parts[pfrpc_CENTROID].pos,  fts.imageCentroid );
    dv2fset( parts[pfrpc_RIGHTHAND].pos, fts.imageRightHand );
    dv2fset( parts[pfrpc_LEFTHAND].pos,  fts.imageLeftHand );
    dv2fset( parts[pfrpc_HEAD].pos,      fts.imageHead );
    dv2fset( parts[pfrpc_RIGHTFOOT].pos, fts.imageRightFoot );
    dv2fset( parts[pfrpc_LEFTFOOT].pos,  fts.imageLeftFoot );

    parts[pfrpc_CENTROID].valid  = TRUE;
    parts[pfrpc_RIGHTHAND].valid = fts.rightHandValid;
    parts[pfrpc_LEFTHAND].valid  = fts.leftHandValid;
    parts[pfrpc_HEAD].valid      = fts.headValid;
    parts[pfrpc_RIGHTFOOT].valid = fts.rightFootValid;
    parts[pfrpc_LEFTFOOT].valid  = fts.leftFootValid;

    if( opts->kalman )
       lite::kalmanSemaphore.remoteUNLOCK();
    else
       lite::getFeatures().semaphore.remoteUNLOCK();

    if (Gaze_valid) {
	rfeats.gaze_pos.x = Gaze.x;
	rfeats.gaze_pos.y = 0;
    } else {
	rfeats.gaze_pos.x = rfeats.gaze_pos.y = 0;
    }
    
    r.pfinfof2_u.feats = &rfeats;
    return r;
}

static pfinfocc2 getcc2d(pfopts *opts) 
{
  int len ;
  pfinfocc2 r;
  static pfinfocc2_s classes[MAXCLASSES];
  r.pfinfocc2_u.parts.parts_len = 0;
  r.pfinfocc2_u.parts.parts_val = NULL;
  if (!(r.opt = opts->cc2d)) return (r);
    
  {
    lite::getFeatures().semaphore.remoteLOCK();
    
    _feature_data& fts = lite::getFeatures().getLatest();
    
    len = fts.numColorClasses;
    for(int i=0; i<len; i++)
      {
	dv3fset( classes[i].cmean, fts.colorclasses2d[i].cmean, NULL );
	dv2fset( classes[i].pos,   fts.colorclasses2d[i].gmean );
	dv2fset( classes[i].gL,    fts.colorclasses2d[i].gL );
	dv2fset( classes[i].gS[0], fts.colorclasses2d[i].gS[0] );
	dv2fset( classes[i].gS[1], fts.colorclasses2d[i].gS[1] );
      }
    
    lite::getFeatures().semaphore.remoteUNLOCK();
  }
  
  r.pfinfocc2_u.parts.parts_len = len;
  r.pfinfocc2_u.parts.parts_val = classes;
  
  return (r);
}

static pfinfocc3 getcc3d(pfopts *opts) 
{
  int len ;
  pfinfocc3 r;
  static pfinfocc3_s classes[MAXCLASSES];
  r.pfinfocc3_u.parts.parts_len = 0;
  r.pfinfocc3_u.parts.parts_val = NULL;
  if (!(r.opt = opts->cc3d)) return (r);
    
  {
    lite::getFeatures().semaphore.remoteLOCK();
    
    _feature_data& fts = lite::getFeatures().getLatest();
    
    len = fts.numColorClasses;

    for(int i=0; i<len; i++)
      {
	dv3fset( classes[i].cmean, fts.colorclasses3d[i].cmean, NULL );
	dv3fset( classes[i].pos,   fts.colorclasses3d[i].gmean, opts);
	dv3fset( classes[i].gL,    fts.colorclasses3d[i].gL,    opts );
	dv3fset( classes[i].gS[0], fts.colorclasses3d[i].gS[0], opts );
	dv3fset( classes[i].gS[1], fts.colorclasses3d[i].gS[1], opts );
	dv3fset( classes[i].gS[2], fts.colorclasses3d[i].gS[2], opts );
      }	

    lite::getFeatures().semaphore.remoteUNLOCK();
  }
  
  
  r.pfinfocc3_u.parts.parts_len = len;
  r.pfinfocc3_u.parts.parts_val = classes;
  
  return (r);
}

// ------------------------------------------------
// Server-side stubs 

void *
pfinit_2( void * , struct svc_req * ) 
{
	int null=0;
	return (void*)&null;
}

void *
pfexit_2( void *, struct svc_req * ) 
{
	int null=0;
	return (void*)&null;
}

void *
pfsetgaze_2( pfv3f *g, struct svc_req * ) 
{
      Gaze = *g;
      Gaze_valid = 1;
      int null=0;
      return (void*)&null;
}


extern int SEGMENT, SEGMENT_TRIGGER;
void *
pftrigger_2( int *a, struct svc_req * ) 
{
	if (*a) {
		SEGMENT = 0;
		SEGMENT_TRIGGER = 1;
	} else { 
		SEGMENT = 1;
	}
	int null=0;
	return (void*)&null;
}


extern int pf_hits;

static pfinfo resultperson;
pfinfo *
pf_2( pfopts *opts, struct svc_req *req ) 
{
  extern int bsMainGetFrameNumber();

  pfinfo *r=&resultperson;
  if (opts->sync) {
    cache_pf_request(opts,req);
    return NULL;
  }
  
  pf_hits++;
  
  r->n  = bsMainGetFrameNumber();

  r->f2 = getf2(opts);
  r->f3 = getf3(opts); 
  r->g = getg(opts); 
  r->c2 = getc2(opts);
  r->c3 = getc3(opts);
  r->cc2d = getcc2d(opts);
  r->cc3d = getcc3d(opts);
  r->texture = gettexture(opts);
  
  return &resultperson;
}

static pfv2f scrsize;
pfv2f *
pfscrsize_2( void *, struct svc_req * )
{
  int x,y;

  bsClassifyGetScreenSize(&x, &y);

  scrsize.x = (float) x;
  scrsize.y = (float) y;
  
  return &scrsize;
}

static pfv3f dimensions;

pfv3f * 
pfuserdimensions_2 ( pfopts *opts, struct svc_req * ) 
{
  if( opts->kalman )
    lite::kalmanSemaphore.remoteLOCK();
  else
    lite::getFeatures().semaphore.remoteLOCK();
  
  if ( lite::PersonDimensionsValid() )
    dv3fset( dimensions, lite::PersonDimensions(), opts );
  else
    dimensions.x = dimensions.y =  dimensions.z = 0.0;
  
  if( opts->kalman )
    lite::kalmanSemaphore.remoteUNLOCK();
  else
    lite::getFeatures().semaphore.remoteUNLOCK();

  return (&dimensions);
}
	
