/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#define LIITE

#include <stream.h>
#include <rpc/rpc.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <signal.h>
#include <sys/time.h>
#include <unistd.h>
#include <limits.h>
#include <bstring.h>

#include "pfinder_svc.h"
#include "lite.h++"

#include "pfTextureMap.h++"

// pf_global_texture_map is defined in pfinder-main.c++
extern pfTextureMap *pf_global_texture_map;

static const float ALSCALE = 1.5;

extern void cache_pf_request(pfopts *opts,struct svc_req *req);

// ------------------------------------------------
// Server-side stubs 

void *
pfinit_1( void * , struct svc_req * ) 
{
	int null=0;
	return (void*)&null;
}

void *
pfexit_1( void *, struct svc_req * ) 
{
	int null=0;
	return (void*)&null;
}

int Gaze_valid = 0;
pfv3f Gaze;
void *
pfsetgaze_1( pfv3f *g, struct svc_req * ) 
{
      Gaze = *g;
      Gaze_valid = 1;
      int null=0;
      return (void*)&null;
}


extern int SEGMENT, SEGMENT_TRIGGER;
void *
pftrigger_1( int *a, struct svc_req * ) 
{
	if (*a) {
		SEGMENT = 0;
		SEGMENT_TRIGGER = 1;
	} else { 
		SEGMENT = 1;
	}
	int null=0;
	return (void*)&null;
}


inline
void
dv3fset( pfv3f& a, float* b, pfopts *opts )
{
    a.x = b[0];
    a.y = b[1];
    a.z = b[2];
    if( opts && !opts->alive_coords )
       {
	   a.x *= ALSCALE;
	   a.y *= ALSCALE;
	   a.z *= ALSCALE;
       }
}

inline
void
dv2fset( pfv2f& a, float* b )
{
    a.x = b[0] - .5;
    a.y = b[1] - .5;
    a.y *= lite::aspectRatio(); // height/width
}

pffeat3d
getf3( pfopts *opts)
{
    pffeat3d r;
    static pffeat3d_s rfeats;
    r.pffeat3d_u.feats = NULL;
    if (!(r.opt = opts->f3)) return r;
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteLOCK();
    else
       lite::getFeatures().semaphore.remoteLOCK();

    _feature_data& fts = 
       (opts->kalman) ? (lite::FeatureDataKalman()) : (lite::getFeatures().getLatest());

    dv3fset( rfeats.left_hand, fts.userplaneLeftHand, opts );
    dv3fset( rfeats.right_hand, fts.userplaneRightHand, opts );
    dv3fset( rfeats.head, fts.userplaneHead, opts );
    dv3fset( rfeats.centroid, fts.userplaneCentroid, opts );
    rfeats.left_hand_valid = fts.leftHandValid;
    rfeats.right_hand_valid = fts.rightHandValid;
    rfeats.head_valid = fts.headValid;
    
    if (Gaze_valid) {
	rfeats.gaze_pos = Gaze;
    } else {
#define TMPGAZE
#ifdef TMPGAZE
	rfeats.gaze_pos.y = rfeats.gaze_pos.z = 0.0;
	rfeats.gaze_pos.x = rfeats.right_hand.x - rfeats.centroid.x;
#else
	rfeats.gaze_pos.x = rfeats.gaze_pos.y = rfeats.gaze_pos.z = 0.0;
#endif
    }
    dv3fset( rfeats.normal_size, lite::PersonDimensions(), opts );
    // PersonDimensions = ( width, height, ?? )
    rfeats.normal_valid = lite::PersonDimensionsValid();
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteUNLOCK();
    else
       lite::getFeatures().semaphore.remoteUNLOCK();

    r.pffeat3d_u.feats = &rfeats;
    return r;
}

pffeat2d getf2( pfopts *opts)
{
    pffeat2d r;
    static pffeat2d_s rfeats;
    r.pffeat2d_u.feats = NULL;
    if (!(r.opt = opts->f2)) return r;
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteLOCK();
    else
       lite::getFeatures().semaphore.remoteLOCK();

    _feature_data& fts = 
       (opts->kalman) 
       ? (lite::FeatureDataKalman()) 
       : (lite::getFeatures().getLatest());

    dv2fset( rfeats.left_hand, fts.imageLeftHand );
    dv2fset( rfeats.right_hand, fts.imageRightHand );
    dv2fset( rfeats.head, fts.imageHead );
    dv2fset( rfeats.centroid, fts.imageCentroid );
    rfeats.left_hand_valid = fts.leftHandValid;
    rfeats.right_hand_valid = fts.rightHandValid;
    rfeats.head_valid = fts.headValid;
    
    if( opts->kalman )
       lite::kalmanSemaphore.remoteUNLOCK();
    else
       lite::getFeatures().semaphore.remoteUNLOCK();

    if (Gaze_valid) {
	rfeats.gaze_pos.x = Gaze.x;
	rfeats.gaze_pos.y = 0;
    } else {
	rfeats.gaze_pos.x = rfeats.gaze_pos.y = 0;
    }
    
    r.pffeat2d_u.feats = &rfeats;
    return r;
}

pfgest
getg( pfopts *opts) 
{
    pfgest r;
    static pfgest_s rgest;
    r.pfgest_u.bits = NULL;
    if (!(r.opt = opts->g)) return r;

    pfGesture& G = lite::getGestures();

    G.semaphore.remoteLOCK();

    rgest.extend_left = G.extendedLeft();
    rgest.extend_right = G.extendedRight();
    rgest.point_left = G.pointingLeft();
    rgest.point_right = G.pointingRight();
    rgest.hand_moving_left = G.handMovingLeft();
    rgest.hand_moving_right = G.handMovingRight();
    rgest.kick_left = G.lhandOverHead();
    rgest.kick_right = G.rhandOverHead();
    rgest.throw_left = G.throwLeft();
    rgest.throw_right = G.throwRight();
    rgest.standing = G.standing();
    rgest.sitting = G.sitting();
    rgest.bending_over = G.bendingOver();
    rgest.user_exists = lite::exists();

    G.semaphore.remoteUNLOCK();

    r.pfgest_u.bits = &rgest;
    return r;

}

pfcont3d
getc3(pfopts *opts) 
{
    pfcont3d r;
    static pfv3f points[MAXCONTOURLEN];
    r.pfcont3d_u.points.points_val = NULL;
    if (!(r.opt = opts->c3)) return r;
    
    lite::getFeatures().semaphore.remoteLOCK();

    _feature_data& fts = lite::getFeatures().getLatest();

    for( int i = 0; i < fts.numContourPoints; i++ )
       dv3fset( points[i], fts.userplaneContour+3*i, opts );
    r.pfcont3d_u.points.points_len = fts.numContourPoints;
    r.pfcont3d_u.points.points_val = points;

    lite::getFeatures().semaphore.remoteUNLOCK();

    return r;
}


pfcc2d
getcc2d(pfopts *opts) 
{
  int len ;
  pfcc2d r;
  static pfcc2d_s classes[MAXCLASSES];
  r.pfcc2d_u.cc.cc_val = NULL;
  if (!(r.opt = opts->cc2d)) return (r);
    
  {
    lite::getFeatures().semaphore.remoteLOCK();
    
    _feature_data& fts = lite::getFeatures().getLatest();
    
    len = fts.numColorClasses;
    for(int i=0; i<len; i++)
      {
	memcpy(classes[i].cmean, 
	       fts.colorclasses2d[i].cmean,
	       3*sizeof(float));
	
	memcpy(classes[i].gmean, 
	       fts.colorclasses2d[i].gmean,
	       2*sizeof(float));
	
	memcpy(classes[i].gL, 
	       fts.colorclasses2d[i].gL,
	       2*sizeof(float));
	
	classes[i].gS[0].x = fts.colorclasses2d[i].gS[0][0];
	classes[i].gS[0].y = fts.colorclasses2d[i].gS[0][1];
	classes[i].gS[1].x = fts.colorclasses2d[i].gS[1][0];
	classes[i].gS[1].y = fts.colorclasses2d[i].gS[1][1];
      }
    
    lite::getFeatures().semaphore.remoteUNLOCK();
  }
  
  r.pfcc2d_u.cc.cc_len = len;
  r.pfcc2d_u.cc.cc_val = classes;
  
  return (r);
}

pfcc3d
getcc3d(pfopts *opts) 
{
  int len ;
  pfcc3d r;
  static pfcc3d_s classes[MAXCLASSES];
  r.pfcc3d_u.cc.cc_val = NULL;
  if (!(r.opt = opts->cc3d)) return (r);
    
  {
    lite::getFeatures().semaphore.remoteLOCK();
    
    _feature_data& fts = lite::getFeatures().getLatest();
    
    len = fts.numColorClasses;
    for(int i=0; i<len; i++)
      {
	memcpy(classes[i].cmean, 
	       fts.colorclasses3d[i].cmean,
	       3*sizeof(float));
	
	memcpy(classes[i].gmean, 
	       fts.colorclasses3d[i].gmean,
	       3*sizeof(float));
	
	memcpy(classes[i].gL, 
	       fts.colorclasses3d[i].gL,
	       3*sizeof(float));
	
	classes[i].gS[0].x = fts.colorclasses3d[i].gS[0][0];
	classes[i].gS[0].y = fts.colorclasses3d[i].gS[0][1];
	classes[i].gS[0].z = fts.colorclasses3d[i].gS[0][2];
	classes[i].gS[1].x = fts.colorclasses3d[i].gS[1][0];
	classes[i].gS[1].y = fts.colorclasses3d[i].gS[1][1];
	classes[i].gS[1].z = fts.colorclasses3d[i].gS[1][2];
	classes[i].gS[2].x = fts.colorclasses3d[i].gS[2][0];
	classes[i].gS[2].y = fts.colorclasses3d[i].gS[2][1];
	classes[i].gS[2].z = fts.colorclasses3d[i].gS[2][2];
	
      }
    
    lite::getFeatures().semaphore.remoteUNLOCK();
  }
  
  r.pfcc3d_u.cc.cc_len = len;
  r.pfcc3d_u.cc.cc_val = classes;
  
  return (r);
}


pfcont2d
getc2(pfopts *opts) 
{
    pfcont2d r;
    static pfv2f points[MAXCONTOURLEN];
    r.pfcont2d_u.points.points_val = NULL;
    if (!(r.opt = opts->c2)) return r;

    lite::getFeatures().semaphore.remoteLOCK();

    _feature_data& fts = lite::getFeatures().getLatest();

    for( int i = 0; i < fts.numContourPoints; i++ )
       dv2fset( points[i], fts.imageContour+2*i );
    r.pfcont2d_u.points.points_len = fts.numContourPoints;
    r.pfcont2d_u.points.points_val = points;

    lite::getFeatures().semaphore.remoteUNLOCK();

    return r;
}

pftexture
gettexture(pfopts *opts)
{
  pftexture t;
  static pftexture_s rtexture;
  static unsigned char *outgoing_data = NULL;
  int width, height;

  t.pftexture_u.tex = NULL;
  if (!(t.opt = opts->texture))
    return t;

  if (pf_global_texture_map == NULL)
    return t;

  if (outgoing_data == NULL)
    {
      outgoing_data = new unsigned char[4 * 
					pf_global_texture_map->getMaxTextureWidth() *
					pf_global_texture_map->getMaxTextureHeight()];
    }
					
  pf_global_texture_map->semaphore.remoteLOCK();

  width = pf_global_texture_map->getTextureWidth();
  height = pf_global_texture_map->getTextureHeight();

  bcopy(pf_global_texture_map->getTextureData(),
	outgoing_data,
	4 * width * height);

  rtexture.width = width;
  rtexture.height = height;
  rtexture.texture.texture_len = 4 * width * height;
  rtexture.texture.texture_val = outgoing_data;
  
  pf_global_texture_map->semaphore.remoteUNLOCK();

  t.pftexture_u.tex = &rtexture;
  return t;
}

int pf_hits = 0;

static pfperson resultperson;
pfperson *
pf_1( pfopts *opts, struct svc_req *req ) 
{
  extern int bsMainGetFrameNumber();

  pfperson *r=&resultperson;
  if (opts->sync) {
    cache_pf_request(opts,req);
    return NULL;
  }
  
  pf_hits++;
  
  r->n  = bsMainGetFrameNumber();

  r->f2 = getf2(opts);
  r->f3 = getf3(opts); 
  r->g = getg(opts); 
  r->c2 = getc2(opts);
  r->c3 = getc3(opts);
  r->cc2d = getcc2d(opts);
  r->cc3d = getcc3d(opts);
  r->texture = gettexture(opts);
  
  return &resultperson;
}

pfv2f scrsize;
pfv2f *
pfscrsize_1( void *, struct svc_req * )
{
  int x,y;

  bsClassifyGetScreenSize(&x, &y);

  scrsize.x = (float) x;
  scrsize.y = (float) y;
  
  return &scrsize;
}

float height;
float  *
pfuserheight_1( void *, struct svc_req * )
{
  _feature_data& fts = lite::getFeatures().getLatest();
  height = fts.userheight;
  return (&height);
}
	

// ------------------------------------------------
void
serverThread( void * )
{
  signal(SIGTERM,SIG_DFL);
  rpc_init();
  svc_run();
}
int child_pid;
void 
startRpcServer( void) {
      unsigned int inh = PR_SALL | PR_SETEXITSIG;
  child_pid = sproc( serverThread, inh );
  if( -1 == child_pid )
     perror( "pfinder: sproc failed" ), exit(1);
}

void
stopRpcServer(void) {
      signal(SIGTERM,SIG_IGN);
      kill(child_pid, SIGTERM);
}

