/*
 * pfinder.x
 *
 * T. Darrell 5/95 
 *
 */

/***************************************************************************
 * 
 * Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

const YES=1;
const NO=0;
const pfrpc_CENTROID= 0;
const pfrpc_RIGHTHAND=1;
const pfrpc_LEFTHAND= 2;
const pfrpc_HEAD=     3;
const pfrpc_RIGHTFOOT=4;
const pfrpc_LEFTFOOT= 5;
const pfrpc_CHEST=    6;
const pfrpc_THIGHS=   7;
const pfrpc_WAIST=    8;

struct pfopts {
	bool sync; /* synchronous? (reply after next full frame) */

			/* known bug: if a client crashes in the middle of a 
			   sync RPC call to pfinder, it may crash the server. 
			   use sync only if needed. */

	bool alive_coords; /* alive coords or world coords? */
	bool kalman; /* kalman filtered or raw values? */
	bool f2;
	bool f3;
	bool c2;
	bool c3;
	bool g;
	bool cc2d;		/* color classes: m and (L,Q) */
	bool cc3d;		/* projected into alive coords */
	bool texture; /* Send associated texture map of video?
			 -- kbrussel 3/24/96 */
};

struct pfv2f {
	float x;
	float y;
};

struct pfv3f {
	float x;
	float y;
	float z;
};

struct pfcc2d_s {
  float cmean[3];		/* color (RGB) mean */
  float gmean[2];		/* geometric (screen) mean normalized */
  float gL[2];			/* l0 l1  (normalized) */
  pfv2f gS[2];			/* s0x s0y s1x s1y */
				/* A*li = li*[six siy] */
};

struct pfcc3d_s {
  float cmean[3];		/* same as pfcc2d projected into 3D */
  float gmean[3];		
  float gL[3];			/* room for 3d blobs... but they're */
  pfv3f gS[3];			/* jsut 2d projected onto a plane in 3d */
				/* for now */
};

const MAXCONTOURLEN = 5000;
const MAXCLASSES    = 50;

struct pfgest_s {
    bool extend_left;
    bool extend_right;
    bool point_left;
    bool point_right;
    bool hand_moving_left;
    bool hand_moving_right;

    bool kick_left;
    bool kick_right;

    bool throw_left;
    bool throw_right;

    bool standing;
    bool sitting;
    bool bending_over;
    bool user_exists;
};


struct pffeat2d_s
{
  pfv2f left_hand;
  pfv2f right_hand;
  pfv2f head;
  pfv2f centroid;

  bool left_hand_valid;
  bool right_hand_valid;
  bool head_valid;
    
  pfv2f gaze_pos;
};


struct pffeat3d_s
{
  pfv3f left_hand;
  pfv3f right_hand;
  pfv3f head;
  pfv3f centroid;

  bool left_hand_valid;
  bool right_hand_valid;
  bool head_valid;
    
  pfv3f gaze_pos;

  pfv3f normal_size;  /* 0 is third coordinate: width, height */
  bool normal_valid;
};

/* Structure for sending bounding rectangle of video.
   -- kbrussel 3/24/96 */
struct pftexture_s
{
  int width;
  int height;   /* Only bounding rectangle of person's
		   video is sent */
  unsigned char texture<>;
};

union pfcont2d switch (int opt) {
case YES:
	pfv2f points<MAXCONTOURLEN>;
case NO:
	void;
default:
	void;
};

union pfcont3d switch (int opt) {
case YES:
	pfv3f points<MAXCONTOURLEN>;
case NO:
	void;
default:
	void;
};

union pfcc2d switch (int opt) {
case YES:
	pfcc2d_s cc<MAXCLASSES>;
case NO:
	void;
default:
	void;
};

union pfcc3d switch (int opt) {
case YES:
	pfcc3d_s cc<MAXCLASSES>;
case NO:
	void;
default:
	void;
};

union pfgest switch (int opt) {
case YES:
	pfgest_s *bits;
case NO:
	void;
default:
	void;
};

union pffeat2d switch (int opt) {
case YES:
	pffeat2d_s *feats;
case NO:
	void;
default:
	void;
};

union pffeat3d switch (int opt) {
case YES:
	pffeat3d_s *feats;
case NO:
	void;
default:
	void;
};

union pftexture switch (int opt) {
case YES:
	pftexture_s *tex;
case NO:
	void;
default:
	void;
};
     
struct pfperson {
  int n;
  
  pffeat2d f2;
  pfcont2d c2;
  
  pffeat3d f3;
  pfcont3d c3;
  
  pfgest g;
  
  pfcc2d cc2d;
  pfcc3d cc3d;

  pftexture texture;
};

struct pfinfocc2_s {
  bool valid;
  pfv3f cmean;		/* color (RGB) mean */
  pfv2f pos;		/* geometric (screen) mean normalized */
  pfv2f gL;		/* eigenvalues (l0 l1) */
  pfv2f gS[2];		/* eigenvectors ([s0x s0y] [s1x s1y]) */
			/* K*li = li*[six siy] */
};

struct pfinfocc3_s {
  bool valid;
  pfv3f cmean;		/* same as pfcc2d projected into 3D */
  pfv3f pos;		
  pfv3f gL;		/* room for 3d blobs... but they're */
  pfv3f gS[3];		/* just 2d projected onto a plane in 3d */
			/* for now */
};

struct pfinfof2_e {
  bool valid;
  pfv2f pos;
};

struct pfinfof3_e {
  bool valid;
  pfv3f pos;
};

const MAXCONTOURLEN = 5000;
const MAXCLASSES    = 50;
const MAXFEATS      = 10;

struct pfinfof2_s
{
	pfinfof2_e parts<MAXCLASSES>;
	pfv2f gaze_pos;
};

struct pfinfof3_s
{
	pfinfof3_e parts<MAXCLASSES>;
	pfv3f gaze_pos;
};

union pfinfoc2 switch (int opt) {
case YES:
	pfv2f points<MAXCONTOURLEN>;
case NO:
	void;
default:
	void;
};

union pfinfoc3 switch (int opt) {
case YES:
	pfv3f points<MAXCONTOURLEN>;
case NO:
	void;
default:
	void;
};

union pfinfocc2 switch (int opt) {
case YES:
	pfinfocc2_s parts<MAXCLASSES>;
case NO:
	void;
default:
	void;
};

union pfinfocc3 switch (int opt) {
case YES:
	pfinfocc3_s parts<MAXCLASSES>;
case NO:
	void;
default:
	void;
};

union pfinfof2 switch (int opt) {
case YES:
	pfinfof2_s *feats;
case NO:
	void;
default:
	void;
};

union pfinfof3 switch (int opt) {
case YES:
	pfinfof3_s *feats;
case NO:
	void;
default:
	void;
};

struct pfinfo {
  int n;
  
  pfinfof2 f2;
  pfinfof3 f3;

  pfcont2d c2;
  pfcont3d c3;
  
  pfgest  g;
  
  pfinfocc2 cc2d;
  pfinfocc3 cc3d;

  pftexture texture;
};

program PFINDERPROG {
	version PFINDERVERS {
		void pfinit( void ) = 11;
		void pftrigger( int ) = 10;
		pfperson pf( pfopts ) = 12;
		void pfexit( void ) = 13;

		void pfsetgaze( pfv3f ) = 14;

		pfv2f pfscrsize( void ) = 15;

		float pfuserheight( void ) = 16;
	} = 1;
	version PFINDERVERS2 {
		void pfinit( void ) = 11;
		void pftrigger( int ) = 10;
		pfinfo pf( pfopts ) = 12;
		void pfexit( void ) = 13;
		void pfsetgaze( pfv3f ) = 14;
		pfv2f pfscrsize( void ) = 15;
		pfv3f pfuserdimensions ( pfopts ) = 16;
	} = 2;
} = 0x20000095;



